// Plant waterer
// Levi Cobb code draft
#include <math.h>
#include <Servo.h>
Servo myservo1;
Servo myservo2;
int moisturePin = A0;
int moistureValue = 0;
int threshold = 100;
int servopin1 = 9;
int servopin2 = 10;
int button = 4;
int buttonval;

//Tempeture reader
int tempsens = A1;
float tempdata;
float tempC;


int h0 = 3; // CONSTANT initial height of water; full tank
int tf = 50; // CONSTANT time for tank to fully empty, determined experimentally
float dh = .2; // CONSTANT amount of height to decrease each time water is released, should be determined based on how much VOLUME we want released every time
float h = h0; // VARIABLE height is initally h0
float t = 0; // VARIABLE total time that water has been running; think of it as clock that runs only when gate is open
float dt = 0; // VARIABLE time for gate to be held open, this changes depending on the height of the water

int timetoopengate = 1;

long lastWateringTime = -900000;  // VARIABLE saving last watering time, initally -30000 so that the moisture sensor can activate immediately after program starts
long watercooldown =  900000;  // CONSTANT cooldown time for watering cycles, 15 minutes in milliseconds 
long tempcooldown = 60000; //one minute minimum between a change in the roof

void setup()
{
  
  
  myservo1.attach(servopin1);
  myservo2.attach(servopin2);  
  pinMode(button, INPUT_PULLUP);
  
  Serial.begin(9600);
  
  Serial.print("h = ");
  Serial.println(h);
  Serial.print("dt = ");
  Serial.println(dt);
  
  myservo1.write(0);
  myservo2.write(0);
}

void loop()
{
  moistureValue = analogRead(moisturePin);
  
  Serial.print("Soil Moisture Value: ");
  Serial.println(moistureValue);
  
  
  
  buttonval = digitalRead(button);
    
    if (buttonval == LOW) {
    //button resets the tank level values after the tank is filled (reset button)
    h = h0;
    t = 0;
    dt = 0;
    delay(1000);
  }
  
  
  //Temp Data
  tempdata = analogRead(tempsens)*0.004882813; //temp in Voltage
  tempdata = (tempdata - 0.5)*100.0; //temp in degrees C
  Serial.print("The temperature is: ");
  Serial.print(tempdata);
  Serial.println("C.");
  
  long currentTime = millis();  // Get current time in milliseconds
  
  if (currentTime - lastWateringTime > watercooldown) {

    if (moistureValue < threshold) {

      h = h - dh; // sets height to what height will be after water is poured
      t = t + dt; // finds total time to reduce the height from h0 to the current height
      dt = (-tf*(sqrt(h)-sqrt(h0))/sqrt(h0) - t); // finds time difference to get from the previous height to the current height
      // dt is how long the gate will be open to let water out

      Serial.print("h = ");
      Serial.println(h);
      Serial.print("dt = ");
      Serial.println(dt);

      myservo1.write(180);

      delay(dt*1000); //length of time for gate to be open

      myservo1.write(0);

      lastWateringTime = millis(); // saves last watering time for the cooldown check

    }

  }
  
  if (tempdata>32) {

  
    myservo2.write(90);
  }
    else if(tempdata<30) {
      
      myservo2.write(0);
      
  }
  
  delay(1000);

}

